unit ufrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, XPMan, StdCtrls;

type
  TfrmMain = class(TForm)
    XPManifest1: TXPManifest;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    cmbPuerto: TComboBox;
    btnSaveCOM: TButton;
    cmbComCfg: TComboBox;
    GroupBox3: TGroupBox;
    txtTexto: TEdit;
    btnEnviar: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnSaveCOMClick(Sender: TObject);
    procedure btnEnviarClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    Puerto: THandle;
    procedure SaveCOMSettings;
    procedure Mensaje(Str: String);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses IniFiles, Registry;

procedure GetPortList(List: TStrings);
var
  i: integer;
begin
  List.Clear;
  with TRegistry.Create do
  try
    RootKey:= HKEY_LOCAL_MACHINE;
    Access:= KEY_READ;
    if OpenKey('HARDWARE\DEVICEMAP\SERIALCOMM',FALSE) then
    begin
      GetValueNames(List);
      for i:= List.Count - 1 downto 0 do
        List[i]:= ReadString(List[i]);
      CloseKey;
    end;
  finally
    Free;
  end;
end;

procedure TfrmMain.btnEnviarClick(Sender: TObject);
var
  DCB: TDCB;
  Str: AnsiString;
begin
  // Si no hay nada que enviar salimos
  if txtTexto.Text = EmptyStr  then
    Exit;
  // Si no hemos abierto ya el puerto lo abrimos
  if Puerto = INVALID_HANDLE_VALUE  then
    with cmbPuerto do
      if ItemIndex >= 0 then
        Puerto:= CreateFile(PChar('\\.\' + Items[ItemIndex]), GENERIC_READ or
          GENERIC_WRITE, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0)
      else
        Exit;
  // Si no conseguimos abrir el puerto salimos
  if Puerto = INVALID_HANDLE_VALUE then
  begin
    Mensaje('Error al abrir el puerto serie');
    Exit;
  end;
  // Si conseguimos abrir el puerto deshabilitamos la configuracion
  Groupbox1.Enabled:= FALSE;
  // Configuramos el puerto
  Str:= Trim(cmbComCfg.Text);
  DCB.DCBlength:= Sizeof(DCB);
  if GetCommState(Puerto,DCB) then
    if BuildCommDCB(PAnsiChar(Str),DCB) then
      if SetCommState(Puerto, DCB) then
      begin
        Str:= txtTexto.Text;
        // Enviamos el texto
        if FileWrite(Puerto,PAnsiChar(Str)^,Length(Str)) < 0 then
          Mensaje('Error al escribir en el puerto serie');
        Exit;
      end;
  Mensaje('Los parmetros de configuracin no son correctos');
end;

procedure TfrmMain.btnSaveCOMClick(Sender: TObject);
begin
  SaveCOMSettings;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Si el puerto esta abierto lo cerramos
  if Puerto <> INVALID_HANDLE_VALUE  then
    CloseHandle(Puerto);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  Puerto:= INVALID_HANDLE_VALUE;
  // Creamos una lista de puertos
  GetPortList(cmbPuerto.Items);
  // Y si no encontramos ninguno creamos una lista generica
  if cmbPuerto.Items.Count = 0 then
    for i:= 1 to 99 do
      cmbPuerto.Items.Add(Format('COM%d',[i]));
  // Creamos una lista de posibles configuraciones
  with cmbComCfg.Items do
  begin
    Clear;
    Add('9600,n,8,1');
    Add('9600,n,8,1,p');
    Add('19200,n,8,1');
    Add('19200,n,8,1,p');
    Add('57600,n,8,1');
    Add('57600,n,8,1,p');
  end;
  with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
  try
    with cmbPuerto do
      ItemIndex:= Items.IndexOf(ReadString('COM','PUERTO',EmptyStr));
    cmbComCfg.Text:= Trim(ReadString('COM','PARAM',EmptyStr));
  finally
    Free;
  end;
  // Escogemos el primer valor por defecto
  if (cmbPuerto.ItemIndex < 0) and (cmbPuerto.Items.Count > 0) then
    cmbPuerto.ItemIndex:= 0;
  if cmbComCfg.Text = EmptyStr then
    cmbComCfg.ItemIndex:= 0;
end;

procedure TfrmMain.Mensaje(Str: String);
begin
  Messagebox(Handle,PChar(Str),PChar(Application.Title),
    MB_OK or MB_SETFOREGROUND or MB_TASKMODAL);
end;

procedure TfrmMain.SaveCOMSettings;
var
  DCB: TDCB;
  Str: AnsiString;
begin
  with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
  try
    with cmbPuerto do
      if ItemIndex >= 0 then
        WriteString('COM','PUERTO',Items[ItemIndex])
      else
        WriteString('COM','PUERTO',EmptyStr);
    Str:= Trim(cmbComCfg.Text);
    if Str <> EmptyStr then
    begin
      if BuildCommDCB(PAnsiChar(Str),DCB) then
        WriteString('COM','PARAM',Str)
      else
        Mensaje('Los parmetros de configuracin del puerto no son correctos');
    end else
      WriteString('COM','PARAM',EmptyStr);
  finally
    Free;
  end;
end;

end.
